// Menu.h: Schnittstelle fr die Klasse CMenu.
//
//////////////////////////////////////////////////////////////////////

#if !defined(MENU_HEADER)
#define MENU_HEADER

// Standart Bilbliotheken
#include <stdio.h>
#include <string.h>

// SDL Bibliotheken
#include <sdl.h>
#include <sdl_ttf.h>

// Definitionen
#define MENU_MAX_UMENU				10		// max. Anzahl an Untermens
#define MENU_MAX_ELEMENTE			20		// max. Anzahl an Elementen in einem UM
#define MENU_MAXLEN_TITEL			30		// max. Lnge einens Men-titels
#define MENU_MAXLEN_ELEMENTTEXT		30		// max. Lnge einens Element-Textes
#define MENU_MAXLEN_FILENAME_FONT	30		// max. Lnge des Dateinamens der Schriftart
#define MENU_TEXT_GROESE_STANDART	30		// die Standartgre eines Textes
#define MENU_ZENTRIERT				-1		// Positionsangabe
// Mentasten
#define MENU_TASTE_BACK				0
#define MENU_TASTE_ACTION			1
#define MENU_TASTE_UP				2
#define MENU_TASTE_DOWN				3
// Elementtypen
#define MENU_ETYP_LEER				0
#define MENU_ETYP_TEXT				1
#define MENU_ETYP_TITEL				2
#define MENU_ETYP_BILD				3
#define MENU_ETYP_RECT				4
// ElementActionen
#define MENU_EACTION_KEINE			0
#define MENU_EACTION_UMENU			3
#define MENU_EACTION_FUNKTION		2


// kurze Klassen bersicht
class CMenu_Element;
class CMenu_Element_Text;
class CMenu_Element_Titel;
class CMenu_Element_Bild;
class CMenu_Element_Rect;
class CMenu;
class CHauptMenu;


// So muss eine Funktion aussehen die an Action bergeben wird
typedef void (*menu_action_funkt)(int para);

// Klasse fr das Element in einem Men
class CMenu_Element
{
public:
	CMenu_Element(){typ=MENU_ETYP_LEER;actionPara.atyp=MENU_EACTION_KEINE;};
	virtual ~CMenu_Element(){};

	// ist der Click auf das Element?
	bool click(Uint16 x, Uint16 y);
	
	// das Element zeichnen
	virtual void zeichnen()=0;

	// Status des Elements
	void setActiv(bool a){activ = a;};
	bool getActiv(){return activ;};

	// den Typ des Elementes zurckgeben
	char getTyp(){return typ;};

	// die Aktion des elementes
	int action();
	void setAction(menu_action_funkt pfunkt, int para);
	void setAction(char atyp, int flag, int para);

	static CHauptMenu* hm;

protected:
	// Hier werden die Parameter fr Action gespeichert
	struct strActionPara
	{
		char atyp;
		menu_action_funkt funkt;
		int flag;
		int para;
	} actionPara;
	

	// Position des Elementes
	SDL_Rect pos;
	// der Status des Elementes
	bool activ;
	// der Typ des Elements
	char typ;
};

class CMenu_Element_Text : public CMenu_Element
{
public:
	CMenu_Element_Text(short x, short y, char* text, unsigned char hight);
	~CMenu_Element_Text();
	
	// das Element zeichnen
	void zeichnen();

private:
	// der text im Element
	char text[MENU_MAXLEN_ELEMENTTEXT];
	SDL_Surface* text_surf[2];
};

class CMenu_Element_Titel : public CMenu_Element
{
public:
	CMenu_Element_Titel(short x, short y, char* text, unsigned char hight);
	~CMenu_Element_Titel();
	
	// das Element zeichnen
	void zeichnen();


private:
	// der text im Element
	char text[MENU_MAXLEN_TITEL];
	SDL_Surface* text_surf;
};

class CMenu_Element_Bild : public CMenu_Element
{
public:
	CMenu_Element_Bild(short x, short y, char* datai);
	~CMenu_Element_Bild();
	
	// das Element zeichnen
	void zeichnen();

private:
	SDL_Surface* bild;
};

class CMenu_Element_Rect : public CMenu_Element
{
public:
	CMenu_Element_Rect(short x, short y, Uint16 w,Uint16 h, Uint8 r=0, Uint8 g=0, Uint8 b=0);
	virtual ~CMenu_Element_Rect(){};
	
	// das Element zeichnen
	void zeichnen();

private:
	Uint32 farbe;
};


// Klasse fr die Strucktur eines Standart Mens 
class CMenu
{
public:
	// Konstr./Destr.
	CMenu(SDL_Surface* pscreen, char* fontname);
	virtual ~CMenu();

	// Zeichnen und Zugriff	(gibt Elementnr zurck oder 0)
	char keyPress(char taste);
	char click(Uint16 x, Uint16 y);
	void zeichnen();

	// Elemente erzeugen
	// Rckgabe: (-1)->Fehler | (0)->als Hauptelement erzeugt | (x)->Elementnr
	int newText(char* text, short x, short y, unsigned char hight=MENU_TEXT_GROESE_STANDART);
	int newTitel(char* text,unsigned char hight=MENU_TEXT_GROESE_STANDART*2, short x=MENU_ZENTRIERT, short y=MENU_ZENTRIERT);
	int newBild(char* datei, short x=MENU_ZENTRIERT, short y=MENU_ZENTRIERT);
	int newRect(short x, short y, Uint16 w,Uint16 h, Uint8 r, Uint8 g, Uint8 b);
	void setElementActiv(char nr);

	// den Elementen eine Action zuweisen
	void setElementAction(int nr, menu_action_funkt pfunkt, int para);
	void setElementAction(int nr, char atyp, int flag, int para);

	// Schriftfarben
	static SDL_Color text_farbe_activ;
	static SDL_Color text_farbe_inactiv;
	static SDL_Color titel_farbe;
	static void setColor_text_activ(Uint8 r,Uint8 g,Uint8 b);
	static void setColor_text_inactiv(Uint8 r,Uint8 g,Uint8 b);
	static void setColor_titel(Uint8 r,Uint8 g,Uint8 b);

	// Dateiname der zu verwendenen Schriftart
	static char fontname[MENU_MAXLEN_FILENAME_FONT];

	// Surface mit Text in dieser Schrift erzeugen
	static SDL_Surface* create_text(char* text, unsigned char hight, SDL_Color tfarbe);
	
	// Surface des Bildschirmes
	static SDL_Surface* screen;

private:
	// die Pointer zu den Elementen
	CMenu_Element_Bild* hintbild;
	CMenu_Element_Rect* hintrect;
	CMenu_Element_Titel* titel;
	CMenu_Element* element[MENU_MAX_ELEMENTE];
	int element_anz;
	int element_activ;	// hier wird das gerade aktive Textelement gespeichert
};

class CHauptMenu
{
public:
	CHauptMenu(SDL_Surface* pscreen, char* fontname);
	~CHauptMenu();

	// Zeichnen und Zugriff	(gibt Elementnr zurck oder 0)
	char keyPress(char taste);
	char click(Uint16 x, Uint16 y);
	void zeichnen();

	int addUMenu();
	void selectUMenu(int nr){if(nr>=0&&nr<anz_umenu) act_umenu=nr; menu=umenu[nr];};

	// Pointer zum activen Untermen
	CMenu* menu;
	
private:
	// Die Untermens (index 0 -> hauptmen)
	CMenu* umenu[MENU_MAX_UMENU+1];
	int anz_umenu;
	int act_umenu;
};
#endif